from .base_controller import BaseController
from models.chat import Chat
from models.base import Session
from utils.api_service import APIService
import logging
from datetime import datetime

logger = logging.getLogger(__name__)

class ChatController(BaseController):
    @staticmethod
    def process_chats():
        logger.info("Starting to process chats")
        session = Session()

        try:
            current_time = datetime.now().strftime("%Y-%m-%dT%H:%M:%S")
            start_date = "2025-03-29T00:00:00"  # This should probably be dynamic

            logger.debug(f"Fetching chats from {start_date} to {current_time}")
            chats_data = APIService.fetch_chats(start_date, current_time)

            if not chats_data:
                logger.info("No chats found")
                return 0

            logger.info(f"Found {len(chats_data)} chats to process")
            processed_count = 0

            for chat in chats_data:
                try:
                    if not isinstance(chat, dict):
                        logger.error(f"Invalid chat format: {type(chat)}")
                        continue

                    chat_id = chat.get('chatId')
                    if not chat_id:
                        logger.error(f"Missing chatId in chat data: {chat}")
                        continue

                    def create_or_update_chat():
                        existing_chat = session.query(Chat).filter_by(chat_id=chat_id).with_for_update().first()

                        timestamp = None
                        if chat.get('timestamp'):
                            try:
                                # First try the exact format
                                timestamp = datetime.strptime(chat['timestamp'], '%Y-%m-%dT%H:%M:%S')
                            except ValueError:
                                try:
                                    # If that fails, try parsing with milliseconds
                                    # Handle different millisecond formats (1, 2, or 3 digits)
                                    ts_parts = chat['timestamp'].split('.')
                                    if len(ts_parts) == 2:
                                        # Parse the base part without milliseconds
                                        timestamp = datetime.strptime(ts_parts[0], '%Y-%m-%dT%H:%M:%S')
                                    else:
                                        # If no decimal point, try other formats or use the original
                                        timestamp = datetime.strptime(chat['timestamp'], '%Y-%m-%dT%H:%M:%S')
                                except ValueError as e:
                                    logger.error(f"Invalid timestamp format for chat {chat_id}: {e}")
                                    # Log the problematic timestamp for debugging
                                    logger.debug(f"Problematic timestamp value: {chat['timestamp']}")

                        if not existing_chat:
                            new_chat = Chat(
                                chat_id=chat_id,
                                chat_name=chat.get('name'),
                                is_group=chat.get('isGroup', False),
                                number=chat.get('number'),
                                timestamp=timestamp
                            )
                            session.add(new_chat)
                            logger.debug(f"Created new chat: {chat_id}")
                        else:
                            existing_chat.chat_name = chat.get('name', existing_chat.chat_name)
                            existing_chat.is_group = chat.get('isGroup', existing_chat.is_group)
                            existing_chat.number = chat.get('number', existing_chat.number)
                            if timestamp:
                                existing_chat.timestamp = timestamp
                            logger.debug(f"Updated existing chat: {chat_id}")

                        session.flush()

                    BaseController._execute_with_retry(session, create_or_update_chat)
                    processed_count += 1

                except Exception as e:
                    logger.error(f"Error processing chat {chat.get('chatId', 'unknown')}: {str(e)}")
                    session.rollback()
                    continue

            session.commit()
            logger.info(f"Successfully processed {processed_count} chats")
            return processed_count

        except Exception as e:
            logger.error(f"Error in chat processing: {str(e)}")
            session.rollback()
            return 0
        finally:
            session.close()


